/*
 * All of the documentation and software included in the
 * Alchemy Software is copyrighted by Stanley Kok, Parag
 * Singla, Matthew Richardson, Pedro Domingos, Marc
 * Sumner, Hoifung Poon, and Daniel Lowd.
 * 
 * Copyright [2004-07] Stanley Kok, Parag Singla, Matthew
 * Richardson, Pedro Domingos, Marc Sumner, Hoifung
 * Poon, and Daniel Lowd. All rights reserved.
 * 
 * Contact: Pedro Domingos, University of Washington
 * (pedrod@cs.washington.edu).
 * 
 * Redistribution and use in source and binary forms, with
 * or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above
 * copyright notice, this list of conditions and the
 * following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the
 * above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use
 * of this software must display the following
 * acknowledgment: "This product includes software
 * developed by Stanley Kok, Parag Singla, Matthew
 * Richardson, Pedro Domingos, Marc Sumner, Hoifung
 * Poon, and Daniel Lowd in the Department of Computer Science and
 * Engineering at the University of Washington".
 * 
 * 4. Your publications acknowledge the use or
 * contribution made by the Software to your research
 * using the following citation(s): 
 * Stanley Kok, Parag Singla, Matthew Richardson and
 * Pedro Domingos (2005). "The Alchemy System for
 * Statistical Relational AI", Technical Report,
 * Department of Computer Science and Engineering,
 * University of Washington, Seattle, WA.
 * http://www.cs.washington.edu/ai/alchemy.
 * 
 * 5. Neither the name of the University of Washington nor
 * the names of its contributors may be used to endorse or
 * promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY OF WASHINGTON
 * AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE UNIVERSITY
 * OF WASHINGTON OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#ifndef FLOGPSEUDOLIKELIHOOD_H_AUG_18_2005
#define FLOGPSEUDOLIKELIHOOD_H_AUG_18_2005

#include <fstream>
#include <iostream>
#include <cmath>
#include "array.h"
#include "hashint.h"
#include "rule.h"
#include "timer.h"
  // expl and logl aren't available in cygwin / windows, so use exp and log
#ifndef expl
# define expl exp
# define logl log
#endif

////////////////////////// helper data structures //////////////////////////////

  //LogPseudolikelihood is used in weight learning which makes the closed-world
  //assumption, so there is no ground predicate whose truth value is UNKNOWN
struct IndexAndCount 
{
  IndexAndCount() : index(NULL), count(0) {}
  IndexAndCount(int* const & i, const double& c) : index(i), count(c) {}
  int* index;
  double count;
};


class PseudoLogLikelihood{

 public:
 PseudoLogLikelihood() : priorMeans_(NULL), priorStdDevs_(NULL)
    {
      jDebug = false;
      //curPredPLL =new Array<double>;
      //curPredGrad = new Array<Array<double>*>;
      counts = NULL;
      spy = false;
      spyGround = 2;
      debug =false;
      fastPll = true;
    }

  ~PseudoLogLikelihood(){
   

   curPredGrad.deleteItemsAndClear();

   dataIdx.deleteItemsAndClear();
   
   perAtomPll->deleteItemsAndClear();
   delete perAtomPll;
   for(int ix = 0; ix < counts->size(); ix++){
     Array<Array<IndexAndCount*>*>* tmp1 = (*counts)[ix];
     for(int jx = 0; jx < tmp1->size(); jx++){
       Array<IndexAndCount*>* tmp2 = (*tmp1)[jx];
       //tmp2->deleteItemsAndClear();
       for(int kx = 0; kx < tmp2->size(); kx++){
	 if ((*tmp2)[kx]){
	   (*tmp2)[kx] = NULL;
	 }
       }
     }
     tmp1->deleteItemsAndClear();
   }
   satisfied.deleteItemsAndClear();
   oneOffSat.deleteItemsAndClear();
   counts->deleteItemsAndClear();
   delete counts;
  }

  void setFastPLL(bool val){
    fastPll = val;
  }
  void deleteCounts(){
    if (counts){
      for(int ix = 0; ix < counts->size(); ix++){
	Array<Array<IndexAndCount*>*>* tmp = (*counts)[ix];
	for (int jx = 0; jx < tmp->size(); jx++){
	  if ((*tmp)[jx]){
	    Array<IndexAndCount*>* tmp1 = (*tmp)[jx];
	    if (tmp1){
	      tmp1->deleteItemsAndClear();
	    }
	  }
	}
	tmp->deleteItemsAndClear();
      }
    }
    counts->deleteItemsAndClear();
    delete counts;
  }

  double getFastValueAndGradient(double* const & gradient, const double* const & wt, const int& arrSize, int idx){
    
    if (idx < 0){
      perPredPllCache.growToSize(counts->size(),0);
      double score =  getValueAndGradient(gradient, wt, arrSize);
      return score;
    }
    else{

      double score = getValueAndGradient(gradient, wt, arrSize, idx);
      return score;
    }
  }
  

  double getValueAndGradient(double* const & gradient, const double* const & wt, const int& arrSize, int idx)
  {
    //double wpll = 0;
    memset(gradient, 0, arrSize*sizeof(double));
    long double wpll = 0; // weighted pseudo-log-likelihood
    long double* perPredGrad = new long double[arrSize];
    //cout << "COunts size: " << counts->size() << endl;
    for (int p = 0; p < counts->size(); p++) // for each predicate
    {
      if (p == 4){
	//spy = true;
      }
      else{
	spy = false;
      }
      int compute = 0;
      if (ignoreSet.contains(p)){
	compute = 1;
      }
      else if (addSet.contains(p)){
	compute = 2;
      }
 
      //commented out: even though pred does not appear in any clause, each
      //of its groundings contribute -ln(2) to the wpll
 
      long double oldPll = 0;
      long double perPredPll = 0;
      memset(perPredGrad, 0, arrSize*sizeof(long double));
      Array<Array<IndexAndCount*>*>* gndingsToClauseIndexesAndCounts = (*counts)[p];
      if (fastPll && compute == 0){
	perPredPll = perPredPllCache[p];
      }
      //if pred p appears in one or more clauses
      else if ((*counts)[p] != NULL){
	//use all groundings of predicate
	int numGnds = gndingsToClauseIndexesAndCounts->size();
	//assert(numGnds == (*((*numGndings_)[domainIdx]))[p]);
	  if (compute == 0){
	    for (int g = 0; g < numGnds; g++){ // for each grounding
	      
	      computePerPredPllAndGrad((*gndingsToClauseIndexesAndCounts)[g], g,
				       wt, perPredPll, perPredGrad,  p, (*exWt)[g]);

	    }
	  }
	  else if (compute == 1){
	    for (int g = 0; g < numGnds; g++){ // for each grounding
	      computePerPredPllAndGrad((*gndingsToClauseIndexesAndCounts)[g], g, wt, perPredPll,
				       perPredGrad, true, idx, NULL, p, (*exWt)[g]); 
	    }
	  }
	  else{
	    for(int modIdx = 0; modIdx < modifiedPositions->size(); modIdx++){
	      int g = (*modifiedPositions)[modIdx];
	      oldPll += (*((*perAtomPll)[p]))[g];
	      
	      computePerPredPllAndGrad((*gndingsToClauseIndexesAndCounts)[g],  g, wt, 
				       perPredPll, perPredGrad, false, idx, 
				       satisfied[idx], p, (*exWt)[g]);
	      
	      if (jDebug){
		cout << "P " << p << " g " << g << " " << (*((*perAtomPll)[p]))[g] << " " << perPredPll << endl;
	      }
	    }
	    int varPosition = positionIndex->find(p);

            IntHashArray* updateGround = (*oneOffIndexes)[varPosition];
            for(int modIdx = 0; modIdx < updateGround->size(); modIdx++){
              int g = (*updateGround)[modIdx];
              oldPll+= (*((*perAtomPll)[p]))[g];
	      
	      computePerPredPllAndGrad((*gndingsToClauseIndexesAndCounts)[g], 
				       g, wt, perPredPll, perPredGrad, false, idx, 
				       oneOffSat[idx], p, (*exWt)[g]);
	      if (jDebug){
		cout << "P " << p << " g " << g << " " << (*((*perAtomPll)[p]))[g] << " " << perPredPll << endl;
	      }//
	    }
	    long double jScore = perPredPllCache[p] - oldPll + perPredPll;
	    if (jDebug)
	      cout << "P " << p << " pll " << jScore << " cache " << perPredPllCache[p] << " old pll " << oldPll << " " << perPredPll << endl;
	    perPredPll = jScore;
	  }//end of else
	
      }//end if != null
      else{
	//pred p does not appear in any clauses
	perPredPll = 0;
	//perPredPll = (*count)[p]->size() * -log(2);
	//perPredGrad entries are all zeroes
      }
   
      if (debug)
	cout << "Per Pred Pll " << p << ": " << perPredPll << endl;

      //negate the value and gradient here
      wpll -= perPredPll;
      for (int i = 0; i < arrSize; i++) 
	{
	  gradient[i] -= perPredGrad[i]; 
	}
    
    } // for each predicate
    //debug = true;
    if (debug){
	for(int ix = 0; ix < arrSize; ix++){
	  cout << "ix " << ix << " " << gradient[ix] << endl;
	}
    }
    //debug = false;
    delete [] perPredGrad;

    //printWtsGradsWPLL(wt, gradient, arrSize, wpll); //for testing

      // if there are prior penalties
    if (numMeans_ > 0)
    {
      //commented out: numMeans_ can be larger than arrSize,so just consider
      //               the first arrSize items in priorMeans & priorStdDevs
      //assert(numMeans_ == arrSize);
        
      // subtract the gaussian priors
      for (int i = 0; i < arrSize; i++)
      {
	//(*curPredGrad)[i] = gradient[i];
        //since at this point the value and gradient have been negated,
        //add the priors
	if (i == 0){
	  cout << "dev: " << priorStdDevs_[i] << endl;
	}
        wpll += (wt[i]-priorMeans_[i])*(wt[i]-priorMeans_[i])/
          (2*priorStdDevs_[i]*priorStdDevs_[i]);         
        if (i == idx){
	  gradient[i] += (wt[i]-priorMeans_[i])/
	    (priorStdDevs_[i]*priorStdDevs_[i]);
	}
      }
    }
    return wpll; 
  }

  void printEx(){
    for(int ix = 0; ix < counts->size(); ix++){
      Array<Array<IndexAndCount*>*>* tmp = (*counts)[ix];
      Array<IndexAndCount*>* tmp1 = (*tmp)[0];
      cout << "Pred " << ix << ": ";
      for(int jx = 0; jx < tmp1->size(); jx++){
	cout << "(" << *((*tmp1)[jx]->index) << "," << (*tmp1)[jx]->count << ") ";
      }
      cout << endl;
    }
  }

  double getValueAndGradient(double* const & gradient, const double* const & wt, const int& arrSize)
  {
    //cout << "arrSize " << arrSize << endl;
    //double wpll = 0;
      perPredPllCache.growToSize(counts->size(),0);
    memset(gradient, 0, arrSize*sizeof(double));
    long double wpll = 0; // weighted pseudo-log-likelihood
    long double* perPredGrad = new long double[arrSize];

    int numPreds = counts->size();

    for (int p = 0; p < numPreds; p++) // for each predicate
    {
      //cout << "p = " << p << endl;
      //commented out: even though pred does not appear in any clause, each
      //of its groundings contribute -ln(2) to the wpll
      //if ((*gndPredClauseIndexesAndCounts)[p] == NULL) continue;
      if (p == 1){
	//spy = true;
      }
      else{
	spy = false;
      }
      long double perPredPll = 0;
      memset(perPredGrad, 0, arrSize*sizeof(long double));
      Array<Array<IndexAndCount*>*>* gndingsToClauseIndexesAndCounts = (*counts)[p];
        //if pred p appears in one or more clauses
      if ((*counts)[p] != NULL)
      {
	
	//use all groundings of predicate
	int numGnds = gndingsToClauseIndexesAndCounts->size();
	//assert(numGnds == (*((*numGndings_)[domainIdx]))[p]);
	for (int g = 0; g < numGnds; g++) // for each grounding
	  {
	    //double t1 = nt.time();
	    Array<IndexAndCount*>* feats = (*gndingsToClauseIndexesAndCounts)[g];
	    computePerPredPllAndGrad(feats, g, wt, 
				     perPredPll, perPredGrad, p, (*exWt)[g]);
	  }
	//calls += numGnds;
      }
      else
      {   //pred p does not appear in any clauses
	
	perPredPll = 0;
	//perPredPll = (*count)[p]->size() * -log(2);
	
        //perPredGrad entries are all zeroes
      }
      /*
	if (p == 0){
	cout << "P = " << p << " " << perPredPll << " " << wt[0] << " " << wt[1] << endl;
	}
      */
      perPredPllCache[p] = perPredPll;
      //curPredPLL[p] = perPredPll;
      //negate the value and gradient here
      wpll -= perPredPll;
      if (debug)
	cout << "Per Pred Pll " << p << ": " << perPredPll << endl;
      for (int i = 0; i < arrSize; i++) 
        {
	  
          gradient[i] -= perPredGrad[i]; 
        }
      //curPredGrad[p] = predPGrad;
    } // for each predicate
    
    if (debug){
      for(int ix = 0; ix < arrSize; ix++){
	cout << "ix " << ix << " " << gradient[ix] << endl;
      }
    }

    delete [] perPredGrad;

    //printWtsGradsWPLL(wt, gradient, arrSize, wpll); //for testing

      // if there are prior penalties
    if (numMeans_ > 0)
    {
      //commented out: numMeans_ can be larger than arrSize,so just consider
      //               the first arrSize items in priorMeans & priorStdDevs
      //cerr << numMeans_ << " " << arrSize << endl;
      assert(numMeans_ >= arrSize);
      //cout << numMeans_ << " arrSize " << arrSize << endl;
      // subtract the gaussian priors
      for (int i = 0; i < arrSize; i++)
      {
	if (i == 0){
	  cout << "dev: " << priorStdDevs_[i] << endl;
	}
	//(*curPredGrad)[i] = gradient[i];
        //since at this point the value and gradient have been negated,
        //add the priors
        wpll += (wt[i]-priorMeans_[i])*(wt[i]-priorMeans_[i])/
          (2*priorStdDevs_[i]*priorStdDevs_[i]);         
 
        gradient[i] += (wt[i]-priorMeans_[i])/
          (priorStdDevs_[i]*priorStdDevs_[i]);
	
      }
    }
    
    return wpll; 
  }
  void computePerPredPllAndGrad(const Array<IndexAndCount*>*
                                const& clauseIndexesAndCounts,
                                const int& g, const double* const & wt, 
                                long double& perPredPll, 
                                long double * const & perPredGrad,
				int atomId,
				double exampleWt)
  {
      // compute W.(N^bar-N)

      // Changed to logl, expl because otherwise changing
      // order of .mln files changes weight results
    long double wdotn = 0;
    long double lpmb = 0;
    
    //assert(noRepeatedIndex(clauseIndexesAndCounts));
    //reset wdotn, because theise is for all grounds not just one grounding!!
    int numClausesUnifyWith = clauseIndexesAndCounts->size();
    
    for (int i = 0; i < numClausesUnifyWith; i++)
      {

	// grounding g unifies with clause (*clauseIndexesAndCounts)[i].idx
        wdotn += wt[*( (*clauseIndexesAndCounts)[i]->index)] * (*clauseIndexesAndCounts)[i]->count;
	if (spy && g == spyGround){
	  cout << "g " << g << " wt " << wt[*( (*clauseIndexesAndCounts)[i]->index)] << " idx " << *( (*clauseIndexesAndCounts)[i]->index) << " count " << (*clauseIndexesAndCounts)[i]->count << " wdotn " << wdotn << endl;
	}
      }
    
    if (wdotn > 0) lpmb +=  -logl(1+expl(-wdotn))-wdotn;
    else lpmb += -logl(1+expl(wdotn));
    //    cout << "g " << g <<  " lpmb " << lpmb << endl;
    (*((*perAtomPll)[atomId]))[g] = (exampleWt * lpmb);

    perPredPll += (exampleWt * lpmb);
    if (spy){
      cout << "Pll " << g << " " << perPredPll << " " << lpmb << " "; 
      cout << (exampleWt * lpmb) << " exWt " << exampleWt << endl;
    }
    // update gradient
    for (int i = 0; i < numClausesUnifyWith; i++)
      {
        perPredGrad[ *( (*clauseIndexesAndCounts)[i]->index ) ] 
          += (((expl(lpmb)-1) * (*clauseIndexesAndCounts)[i]->count)* exampleWt);
      }
  }

  void setExWt(Array<double>* exWts, Array<Array<int>*>* coverList){
    exWt = exWts;
    clist = coverList;
  }

  void initializeCounts(Array<Array<int>*>* data, Array<Rule*>* rules, Array<int*>* clauseIdx){

    if (counts){
      deleteCounts();
    }
    satisfied.growToSize(rules->size(),NULL);
    oneOffSat.growToSize(rules->size(),NULL);

    perAtomPll = new Array<Array<long double>*>;
    //tmpPerAtomPll = new Array<Array<long double>*>;
 
   
    //variables x datapoints x features
    counts = new Array<Array<Array<IndexAndCount*>*>*>;
    int numFeats = (*data)[0]->size();
    for(int ix = 0; ix < numFeats; ix++){
      Array<Array<IndexAndCount*>*>* tmp = new Array<Array<IndexAndCount*>*>;
      for(int jx =0; jx < data->size(); jx++){
	Array<IndexAndCount*>* tmp1 = new Array<IndexAndCount*>;
	tmp->append(tmp1);
      }
      Array<long double>* tmpD1 = new Array<long double>;
      tmpD1->growToSize(data->size(),0);
      perAtomPll->append(tmpD1);
 

      counts->append(tmp);
    }

    for(int ix = 0; ix < rules->size(); ix++){
      
      IndexAndCount* sat = new IndexAndCount((*clauseIdx)[ix],-1);
      IndexAndCount* oneOff = new IndexAndCount((*clauseIdx)[ix],1);
      satisfied[ix] = sat;
      oneOffSat[ix] = oneOff;
	
      for(int jx = 0; jx < data->size(); jx++){
	if ((*rules)[ix]->covers((*data)[jx])){
	  IntHashArray* vars = (*rules)[ix]->index;
	  for(int kx = 0; kx < vars->size(); kx++){
	    int varId = (*vars)[kx];
	    (*((*counts)[varId]))[jx]->append(sat);
	  }
	}
	else{
	  IntHashArray* vars = (*rules)[ix]->index;
	  for(int kx = 0; kx < vars->size(); kx++){
	    int varId = (*vars)[kx];
	    int tmpVal = (*((*data)[jx]))[varId];
	    (*((*data)[jx]))[varId] = (tmpVal + 1) % 2;
	    if ((*rules)[ix]->covers((*data)[jx])){
	      (*((*counts)[varId]))[jx]->append(oneOff);
	      
	    }
	    (*((*data)[jx]))[varId] = tmpVal;
	  }
	}
      }
    }
    bool printInit = false;//true;//false;
    if (printInit){
      for(int ix = 0; ix < counts->size(); ix++){
	Array<Array<IndexAndCount*>*>* tmp1 = (*counts)[ix];
	for(int jx = 0; jx < 1; jx++){//tmp1->size(); jx++){
	  Array<IndexAndCount*>* tmp2 = (*tmp1)[jx];
	  cout << "ix " << ix << " jx " << jx << " : ";
	  for(int kx = 0; kx < tmp2->size(); kx++){
	    cout << "(" << *((*tmp2)[kx]->index) << "," << (*tmp2)[kx]->count << ") ";
	  }
	  cout << endl;
	}
	
      }
    }
  }

  void updateModel(Rule* rule, int ruleIdx, Array<Array<int>*>* data){
    for(int jx = 0; jx < data->size(); jx++){
      if (rule->covers((*data)[jx])){
	IntHashArray* vars = rule->index;
	for(int kx = 0; kx < vars->size(); kx++){
	  int varId = (*vars)[kx];
	  Array<Array<IndexAndCount*>*>* ex = (*counts)[varId];
	  Array<IndexAndCount*>* clauses = (*ex)[jx];
	  clauses->append(satisfied[ruleIdx]);
	}
      }
      else{
	IntHashArray* vars = rule->index;
	for(int kx = 0; kx < vars->size(); kx++){
	  int varId = (*vars)[kx];
	  int tmpVal = (*((*data)[jx]))[varId];
	  (*((*data)[jx]))[varId] = (tmpVal + 1) % 2;
	  if (rule->covers((*data)[jx])){
	    Array<Array<IndexAndCount*>*>* ex = (*counts)[varId];
	    Array<IndexAndCount*>* clauses = (*ex)[jx];
	    clauses->append(oneOffSat[ruleIdx]);
	  }
	  (*((*data)[jx]))[varId] = tmpVal;
	}
      }
    }
  }



  void addRuleUpdate(int* rulePos){
    satisfied.append(new IndexAndCount(rulePos,-1));
    oneOffSat.append(new IndexAndCount(rulePos,1));
  }

  
  void countNewRule(Rule* rule, Array<Array<int>*>* data){
    //if (sysconf(_SC_AVPHYS_PAGES) < 100){
    //cout << "Pages left: " << sysconf(_SC_AVPHYS_PAGES) << endl;
    //}

    IntHashArray* nvars = rule->index;
    addSet.clear();
    ignoreSet.clear();
    //rule->setOneOff();
    //oneOffIndexes = rule->oneOffPos;
    positionIndex = rule->index;
    for(int ix = 0; ix < nvars->size(); ix++){
      addSet.append((*nvars)[ix]);
    }

    for(int jx = 0; jx < data->size(); jx++){
      if (rule->covers((*data)[jx])){
	/*
	rule->fastCov->append(jx);
	IntHashArray* vars = rule->index;
	for(int kx = 0; kx < vars->size(); kx++){
	  int varId = (*vars)[kx];
	}
	*/
      }
      else{
	IntHashArray* vars = rule->index;
	for(int kx = 0; kx < vars->size(); kx++){
	  int varId = (*vars)[kx];
	  //Array<int>* featureArr = include[varId];
	  int tmpVal = (*((*data)[jx]))[varId];
	  (*((*data)[jx]))[varId] = (tmpVal + 1) % 2;
	  if (rule->covers((*data)[jx])){
	    (*oneOffIndexes)[kx]->append(jx);
	  
	  }

	  (*((*data)[jx]))[varId] = tmpVal;
	}
      }
    }
    //modifiedPositions = rule->fastCov;
  }

  void computePerPredPllAndGrad(const Array<IndexAndCount*>*
                                const& clauseIndexesAndCounts,
                                const int& g, const double* const & wt, 
                                long double& perPredPll, 
                                long double * const & perPredGrad,
				bool ignore,
				int idx,
				IndexAndCount* include,
				int atomId,
				double exampleWt)
  {
      // compute W.(N^bar-N)

      // Changed to logl, expl because otherwise changing
      // order of .mln files changes weight results
    long double wdotn = 0;
    long double lpmb = 0;

    int curPos= 0;   
    bool found = false; 
    int numClausesUnifyWith = clauseIndexesAndCounts->size();
    if (g == 0 && idx == 9){
      //cerr << "g " << g << " idx " << idx << " length " << numClausesUnifyWith << endl;
    }
    for (int i = 0; i < numClausesUnifyWith; i++){

      int tmpIdx =  *( (*clauseIndexesAndCounts)[i]->index );
 
      if (ignore && idx == tmpIdx){
	//clause no longer applies
      }
      else if (tmpIdx == idx){
	found = true;
	//cout << g << " " << *(include->index) << endl;
	wdotn += wt[ *(include->index ) ] * include->count;
	if (spy && g == spyGround){
	  cout << "g " << g << " wt " << wt[*(include->index)] << " idx " << *(include->index) << " count " << include->count << " wdotn " << wdotn << " include " << endl;
	}

      }
      else{
	// grounding g unifies with clause (*clauseIndexesAndCounts)[i].idx
	wdotn += wt[ *( (*clauseIndexesAndCounts)[i]->index ) ] * 
	  (*clauseIndexesAndCounts)[i]->count;
	if (spy && g == spyGround){
	  cout << "g " << g << " wt " << wt[*( (*clauseIndexesAndCounts)[i]->index)] << " idx " << *( (*clauseIndexesAndCounts)[i]->index) << " count " << (*clauseIndexesAndCounts)[i]->count << " wdotn " << wdotn << endl;
	}

      }
    }
    if (!found && include){
      wdotn += wt[ *(include->index ) ] * include->count;
      if (spy && g == spyGround){
	cout << "g " << g << " wt " << wt[*(include->index)] << " idx " << *(include->index) << " count " << include->count << " wdotn " << wdotn << " include " << endl;
      }
    }
    if (wdotn > 0) lpmb += -logl(1+expl(-wdotn))-wdotn;
    else lpmb += -logl(1+expl(wdotn));
    //(*((*tmpPerAtomPll)[atomId]))[g] = exampleWt * lpmb;

    perPredPll += (exampleWt * lpmb);
    if (spy){
      cout << "Pll " << g << " " << perPredPll << " " << lpmb;
      cout << (exampleWt * lpmb) << " " << exampleWt << endl;
    }
    curPos = 0;
    for (int i = 0; i < clauseIndexesAndCounts->size(); i++){
      int tmpIdx =  *( (*clauseIndexesAndCounts)[i]->index );
      if (ignore && tmpIdx == idx){
	
      }
      else if (tmpIdx == idx){
	perPredGrad[ *( include->index ) ] 
	  += (expl(lpmb)-1) * include->count;
      }
    }
    
    if (!found && include){
      perPredGrad[ *( include->index ) ] 
	+= (exampleWt * ((expl(lpmb)-1) * include->count));
    }
  }

    //set numMeans to -1 if there is no prior
  void setMeansStdDevs(const int& arrSize, const double* const & priorMeans, 
                       const double* const & priorStdDevs)
  {
    numMeans_ = arrSize;
    priorMeans_ = priorMeans;
    priorStdDevs_ = priorStdDevs;
    
  }

  void printDev(){
    cout << priorStdDevs_[0] << endl;
  }

  void setj(bool j){
    jDebug = j;
  }
 private:
  bool jDebug;
  bool fastPll;
  bool debug;
  bool spy;
  int spyGround;
  int numMeans_;
  const double* priorMeans_;
  const double* priorStdDevs_;
  Array<IndexAndCount*> satisfied;
  Array<IndexAndCount*> oneOffSat;
  Array<double> perPredPllCache;
  //datapoint
  Array<Array<Array<IndexAndCount*>*>*>* counts;

  Array<double> curPredPLL;
  Array<Array<double>*> curPredGrad;

  Array<double> evalPredPLL;
  Array<Array<double>*> evalPredGrad;

  IntHashArray ignoreSet;
  IntHashArray addSet;
  Array<Array<long double>*>* perAtomPll;
  
  Array<IntHashArray*>* oneOffIndexes;
  IntHashArray* positionIndex;
  IntHashArray* modifiedPositions;
  Array<int> predIdx;
  Array<Array<int>*> dataIdx;

  Array<double>* exWt;
  Array<Array<int>*>* clist;
};
#endif
